@@ -54,6 +54,7 @@ const char *opt_text = NULL;
 gboolean opt_waterfall = FALSE;
 int opt_width = -1;
 int opt_indent = 0;
+gboolean opt_justify = 0;
 int opt_runs = 1;
 PangoAlignment opt_align = PANGO_ALIGN_LEFT;
 PangoEllipsizeMode opt_ellipsize = PANGO_ELLIPSIZE_NONE;
@@ -64,6 +65,7 @@ PangoWrapMode opt_wrap = PANGO_WRAP_WORD_CHAR;
 gboolean opt_wrap_set = FALSE;
 const char *opt_pangorc = NULL;
 const PangoViewer *opt_viewer = NULL;
+const char *opt_language = NULL;
 
 /* Text (or markup) to render */
 static char *text;
@@ -112,6 +114,7 @@ make_layout(PangoContext *context,
 
   pango_layout_set_auto_dir (layout, opt_auto_dir);
   pango_layout_set_ellipsize (layout, opt_ellipsize);
+  pango_layout_set_justify (layout, opt_justify);
 
   font_description = get_font_description ();
   if (size > 0)
@@ -163,8 +166,7 @@ get_options_string (void)
 }
 
 static void
-output_body (PangoContext   *context,
-	     const char     *text,
+output_body (PangoLayout    *layout,
 	     RenderCallback  render_cb,
 	     gpointer        cb_context,
 	     gpointer        cb_data,
@@ -172,7 +174,6 @@ output_body (PangoContext   *context,
 	     int            *height,
 	     gboolean        supports_matrix)
 {
-  PangoLayout *layout;
   PangoRectangle logical_rect;
   int size, start_size, end_size, increment;
   int x = 0, y = 0;
@@ -181,6 +182,7 @@ output_body (PangoContext   *context,
     {
       const PangoMatrix* matrix;
       const PangoMatrix identity = PANGO_MATRIX_INIT;
+      PangoContext *context = pango_layout_get_context (layout);
       matrix = pango_context_get_matrix (context);
       if (matrix)
 	{
@@ -188,6 +190,7 @@ output_body (PangoContext   *context,
 	  y += matrix->y0;
 	}
       pango_context_set_matrix (context, &identity);
+      pango_layout_context_changed (layout);
     }
 
   if (opt_waterfall)
@@ -207,7 +210,14 @@ output_body (PangoContext   *context,
 
   for (size = start_size; size <= end_size; size += increment)
     {
-      layout = make_layout (context, text, size);
+      if (size > 0)
+        {
+	  PangoFontDescription *desc = pango_font_description_copy (pango_layout_get_font_description (layout));
+	  pango_font_description_set_size (desc, size * PANGO_SCALE);
+	  pango_layout_set_font_description (layout, desc);
+	  pango_font_description_free (desc);
+	}
+
       pango_layout_get_extents (layout, NULL, &logical_rect);
 
       if (render_cb)
@@ -216,8 +226,6 @@ output_body (PangoContext   *context,
       *width = MAX (*width, PANGO_PIXELS (logical_rect.x + logical_rect.width));
       *width = MAX (*width, PANGO_PIXELS (pango_layout_get_width (layout)));
       *height += PANGO_PIXELS (logical_rect.height);
-
-      g_object_unref (layout);
     }
 }
 
@@ -266,7 +274,9 @@ do_output (PangoContext     *context,
 
   set_transform (context, transform_cb, cb_context, cb_data, NULL);
 
-  pango_context_set_language (context, pango_language_get_default ());
+  pango_context_set_language (context,
+			      opt_language ? pango_language_from_string (opt_language)
+					   : pango_language_get_default ());
   pango_context_set_base_dir (context,
 			      opt_rtl ? PANGO_DIRECTION_RTL : PANGO_DIRECTION_LTR);
 
@@ -300,10 +310,11 @@ do_output (PangoContext     *context,
   pango_context_set_base_gravity (context, opt_gravity);
   pango_context_set_gravity_hint (context, opt_gravity_hint);
 
+  layout = make_layout (context, text, -1);
+
   set_transform (context, transform_cb, cb_context, cb_data, &matrix);
 
-  output_body (context,
-	       text,
+  output_body (layout,
 	       NULL, NULL, NULL,
 	       &rotated_width, &rotated_height,
 	       supports_matrix);
@@ -320,8 +331,7 @@ do_output (PangoContext     *context,
   set_transform (context, transform_cb, cb_context, cb_data, &matrix);
 
   if (render_cb)
-    output_body (context,
-		 text,
+    output_body (layout,
 		 render_cb, cb_context, cb_data,
 		 &rotated_width, &rotated_height,
 		 supports_matrix);
@@ -339,6 +349,7 @@ do_output (PangoContext     *context,
 
   pango_context_set_matrix (context, orig_matrix);
   pango_matrix_free (orig_matrix);
+  g_object_unref (layout);
 }
 
 static gboolean
@@ -554,7 +565,7 @@ show_version(const char *name,
 void
 parse_options (int argc, char *argv[])
 {
-  gchar *backend_options = backends_to_string();
+  gchar *backend_options = backends_to_string ();
   GOptionFlags backend_flag = backends_get_count () > 1 ? 0 : G_OPTION_FLAG_HIDDEN;
   gchar *backend_desc = backend_description ();
   GOptionEntry entries[] =
@@ -583,6 +594,10 @@ parse_options (int argc, char *argv[])
      "Hinting style",					    "none/auto/full"},
     {"indent",		0, 0, G_OPTION_ARG_INT,				&opt_indent,
      "Width in points to indent paragraphs",			    "points"},
+    {"justify",		0, 0, G_OPTION_ARG_NONE,			&opt_justify,
+     "Align paragraph lines to be justified",			    	NULL},
+    {"language",	0, 0, G_OPTION_ARG_STRING,			&opt_language,
+     "Language to use for font selection",			    "en_US/etc"},
     {"margin",		0, 0, G_OPTION_ARG_INT,				&opt_margin,
      "Set the margin on the output in pixels",			    "pixels"},
     {"markup",		0, 0, G_OPTION_ARG_NONE,			&opt_markup,
