@@ -23,6 +23,7 @@
 #include <stdlib.h>
 
 #include "pango-renderer.h"
+#include "pango-impl-utils.h"
 
 #define N_RENDER_PARTS 4
 
@@ -427,6 +428,7 @@ draw_shaped_glyphs (PangoRenderer    *renderer,
     }
 }
 
+
 /**
  * pango_renderer_draw_layout_line:
  * @renderer: a #PangoRenderer
@@ -490,11 +492,14 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
 
       if (shape_attr)
 	{
-	  ink_rect = shape_attr->ink_rect;
-	  logical_rect = shape_attr->logical_rect;
 	  ink = &ink_rect;
 	  logical = &logical_rect;
-	  glyph_string_width = shape_attr->logical_rect.width;
+          _pango_shape_get_extents (run->glyphs->num_glyphs,
+				    &shape_attr->ink_rect,
+				    &shape_attr->logical_rect,
+				    ink,
+				    logical);
+	  glyph_string_width = logical->width;
 	}
       else
 	{
@@ -517,7 +522,7 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
 
       if (run->item->analysis.flags & PANGO_ANALYSIS_FLAG_CENTERED_BASELINE)
 	{
-	  gboolean is_hinted = (logical_rect.y & logical_rect.height & (PANGO_SCALE - 1)) == 0;
+	  gboolean is_hinted = ((logical_rect.y | logical_rect.height) & (PANGO_SCALE - 1)) == 0;
 	  int adjustment = logical_rect.y + logical_rect.height / 2;
 
 	  if (is_hinted)
